<?php

namespace FSPoster\App\Libraries\xing;

use FSP_GuzzleHttp\Client;
use FSP_GuzzleHttp\Exception\GuzzleException;
use FSP_GuzzleHttp\Psr7\MultipartStream;
use FSPoster\App\Providers\DB;
use FSPoster\App\Providers\Helper;
use FSP_GuzzleHttp\Cookie\CookieJar;

class Xing
{
	/**
	 * @var Client
	 */
	private $client;
	private $cookies;
	private $proxy;
	private $endpoint;
	private $domain = 'www.xing.com';

	public function __construct ( array $cookies, $proxy = null )
	{
		$this->endpoint = 'https://www.xing.com/xing-one/api';
		$this->cookies  = $cookies;
		$this->proxy    = $proxy;

		$this->setClient();
	}

	public function send ( $nodeInfo, $postType, $message, $link, $images )
	{
		$message    = trim( $message );
		$visibility = ( int ) Helper::getOption( 'xing_post_visibility', '1' );
		$data       = [
			'operationName' => 'CreateTextPosting',
			'variables'     => [
				'actorGlobalId'    => 'surn:x-xing:users:user:' . $nodeInfo[ 'profile_id' ],
				'comment'          => '',
				'commentArticleV1' => [
					[
						'articleParagraph' => [
							'text'    => $message,
							'markups' => []
						]
					]
				],
				'visibility' => $visibility === 1 ? 'PUBLIC' : 'PRIVATE',
				'links'      => null,
				'images'     => null,
				'audience'   => $visibility === 1 ? null : 'surn:x-xing:contacts:network:' . $nodeInfo[ 'profile_id' ] . ( $visibility === 3 ? ':same_city' : '' )
			],
			'query'         => 'mutation CreateTextPosting($actorGlobalId: GlobalID!, $comment: String!, $commentArticleV1: [ArticlesCreateArticleBlocksInput!], $visibility: PostingsVisibility, $images: [PostingsCreateImageAttachmentInput!], $links: [PostingsCreateLinkAttachmentInput!], $audience: [GlobalID!]) {  postingsCreatePosting(    input: {actorGlobalId: $actorGlobalId, comment: $comment, commentArticleV1: $commentArticleV1, visibility: $visibility, images: $images, links: $links, audience: $audience}  ) {    success {      id      actorGlobalId      activityId      comment      __typename    }    error {      message      details      __typename    }    __typename  }}'
		];

		if ( $postType === 1 && ! empty( $link ) )
		{
			$data[ 'variables' ][ 'links' ] = [
				[
					'url' => $link
				]
			];

			$cardViewPreview = [
				"operationName" => "SharePreview",
				"variables"     => [ "url" => $link ],
				'query'         => 'query SharePreview($url: URL!) {  viewer {    id    linkPreview(url: $url) {      success {        title        description        sourceDomain        cachedImageUrl        metadata {          sourceActor {            title            subtitle            image            message            __typename          }          __typename        }        __typename      }      __typename    }    __typename  }}'
			];

			$CVPResponse = self::cmd( 'POST', $this->endpoint, json_encode( $cardViewPreview, JSON_UNESCAPED_SLASHES ), [ 'content-type' => 'application/json' ] );

			if ( empty( $CVPResponse[ 'data' ] ) || empty( $CVPResponse[ 'data' ][ 'viewer' ] ) || empty( $CVPResponse[ 'data' ][ 'viewer' ][ 'linkPreview' ] ) || empty( $CVPResponse[ 'data' ][ 'viewer' ][ 'linkPreview' ][ 'success' ] ) )
			{
				return [
					'status' => 'error',
					'error_msg' => fsp__( 'Xing couldn\'t fetch the post URL.' )
				];
			}
		}
		else if ( $postType === 3 && ! empty( $images ) )
		{
			$uploadId = $this->uploadRequest( $images );

			if ( is_array( $uploadId ) && ! empty( $uploadId[ 'error_msg' ] ) )
			{
				return $uploadId;
			}

			$data[ 'variables' ][ 'images' ] = [
				[
					'uploadId' => $uploadId
				]
			];
		}

		$data = json_encode( $data, JSON_UNESCAPED_SLASHES );

		$result = self::cmd( 'POST', $this->endpoint, $data, [ 'content-type' => 'application/json' ]  );

		$returnData = [ 'status' => 'error' ];

		if ( ! is_array( $result ) )
		{
			$returnData [ 'error_msg' ] = fsp__( 'Unknown error!' );
		}
		else if ( ! empty( $result[ 'error_msg' ] ) )
		{
			$returnData = $result;
		}
		else if ( ! empty( $result[ 'errors' ] ) )
		{
			$returnData [ 'error_msg' ] = $result[ 'errors' ][ 0 ][ 'message' ];
		}
		else if ( ! empty( $result[ 'data' ] ) && ! empty( $postingArray = $result[ 'data' ][ 'postingsCreatePosting' ] ) )
		{
			if ( ! empty( $postingArray[ 'error' ] ) )
			{
				$returnData [ 'error_msg' ] = $postingArray[ 'error' ][ 'message' ];
			}
			else
			{
				$returnData = [
					'status' => 'ok',
					'id'     => $postingArray[ 'success' ][ 'activityId' ]
				];
			}
		}

		return $returnData;
	}

	public function sendToGroup ( $nodeInfo, $postType, $title, $message, $link, $image )
	{
		$tokens  = $this->getGroupAuthenticityToken( $nodeInfo[ 'id' ], $nodeInfo[ 'screen_name' ] );
		$forumId = Helper::getCustomSetting( 'xing_group_selected_forum', '', 'node', $nodeInfo[ 'id' ] );

		if ( ! isset( $tokens[ 'authenticity_token' ] ) )
		{
			return $tokens;
		}

		$data      = [
			'authenticity_token'   => $tokens[ 'authenticity_token' ],
			'post[title]'          => $title,
			'post[moderator_news]' => 'off',
			'post[content]'        => $message,
			'next_options'         => 2,
			'post[forum_id]'       => $forumId,
			'stream'               => true
		];

		if ( $postType === 1 )
		{
			$linkId = '';
			$body   = 'link=' . urlencode( $link );

			$response = self::cmd( 'POST', 'https://www.xing.com/communities/media_previews', $body, [ 'Content-Type' => 'application/x-www-form-urlencoded; charset=UTF-8' ] );

			if ( ! is_array( $response ) && empty( $response[ 'html' ] ) )
			{
				return [
					'status'    => 'error',
					'error_msg' => fsp__( 'Xing couldn\'t fetch the post URL. Please check the connection between your Website and Xing.' )
				];
			}

			$previewIdPatter = '/data-preview-id=\"(.*?)\"/';

			preg_match( $previewIdPatter, $response[ 'html' ], $matchedId );

			if ( ! empty( $matchedId[ 1 ] ) )
			{
				$linkId = $matchedId[ 1 ];
			}

			if ( is_array( $linkId ) )
			{
				return $linkId;
			}

			if ( empty( $linkId ) )
			{
				return [
					'status'    => 'error',
					'error_msg' => fsp__( 'Xing couldn\'t fetch the post URL. Please check the connection between your Website and Xing.' )
				];
			}

			$data[ 'post[media_preview_id]' ] = $linkId;
		}
		else if ( $postType === 3 )
		{
			$body     = new MultipartStream( [
				[
					'name'     => 'image',
					'contents' => file_get_contents( $image ),
					'filename' => basename( $image )
				]
			], '----WebKitFormBoundaryRaESXBPMJGyfZQVh' );

			$response = self::cmd( 'POST', 'https://www.xing.com/communities/upload/content_images.json', $body, [ 'Content-Type' => 'multipart/form-data; boundary=----WebKitFormBoundaryRaESXBPMJGyfZQVh' ] );

			if ( empty( $response[ 'image' ] ) )
			{
				return [
					'status'    => 'error',
					'error_msg' => isset( $response[ 'error_msg' ] ) ? $response[ 'error_msg' ] : fsp__( 'Xing couldn\'t upload the post image. Please check the connection between your Website and Xing.' )
				];
			}

			while ( $response[ 'image' ][ 'processing' ] )
			{
				$response = self::cmd( 'GET', 'https://www.xing.com' .  $response[ 'image' ][ 'url'] . '?_=' . round( microtime( true ) * 1000 ), '', [
					'Accept'           => 'text/javascript, text/html, application/xml, text/xml, */*',
					'X-Requested-With' =>'XMLHttpRequest',
					'Connection'       =>'keep-alive',
					'Referer'          => $nodeInfo[ 'screen_name' ]
				] );

				sleep( 2 );
			}

			$data[ 'post[image_attributes][slug]' ] = $response[ 'image' ][ 'slug' ];
			$data[ 'X-represents-preview-image' ]   = $response[ 'image' ][ 'main_url' ];
		}

		$url = 'https://www.xing.com/communities/forums/' . $forumId . '/posts.json';

		$result = self::cmd( 'POST', $url, http_build_query( $data ) );

		if ( ! empty( $result ) && isset( $result[ 'success' ] ) )
		{
			$checkIfCrossPosted = strpos( $result[ 'html' ], 'XING deleted this post due to crossposting.' );

			if ( $checkIfCrossPosted )
			{
				return [
					'status'    => 'error',
					'error_msg' => fsp__( 'XING deleted this post due to crossposting.' )
				];
			}

			return [
				'status' => 'ok',
				'id'     => $result[ 'postPath' ]
			];
		}

		return [
			'status'    => 'error',
			'error_msg' => isset( $result[ 'error_msg' ] ) ? $result[ 'error_msg' ] : fsp__( 'Something went wrong.' )
		];
	}

	public function cmd ( $method, $endpoint, $body, $headers = [], $isArray = true )
	{
		try {
			$response = $this->client->request( $method, $endpoint, [
				'headers' => $headers,
				'body'    => $body
			] )->getBody();
		}
		catch ( GuzzleException $e )
		{
			return [
				'status'    => 'error',
				'error_msg' => fsp__( 'Error! %s', [ $e->getMessage() ] )
			];
		}

		if ( $isArray )
		{
			return json_decode( $response, true );
		}

		return $response;
	}

	public function getAccountData ()
	{
		$data = [
			'operationName' => 'xingFrameQuery',
			'variables'     => [],
			'query'         => 'query xingFrameQuery {  viewer {    id    webTrackingData {      PropHashedUserId      __typename    }    xingId {      academicTitle      birthday      displayName      displayFlag      userFlags {        displayFlag        __typename      }      firstName      gender      globalId      id      lastName      pageName      profileImage(size: [SQUARE_128]) {        url        __typename      }      profileOccupation {        occupationOrg        occupationTitle        __typename      }      occupations {        headline        subline        __typename      }      __typename    }    features {      isXingEmployee      isJobsPoster      isAdmasterUser      isBrandPageCollaborator      isBasic      isPremium      isExecutive      isSales      hasProJobsMembership      isCraUser      isSeatManagerAdmin      showProbusinessInNavigation      showUpsellHint      showJobSeekerBoneyardUpsellHint      showPremiumBoneyardUpsellHint      hasCoachProfile      hasNewSettings      isBrandManagerEditor      __typename    }    featureSwitches    loginState    __typename  }}'
		];

		$accountData = self::cmd( 'POST', $this->endpoint, json_encode( $data ), [ 'content-type' => 'application/json' ] );

		if ( ! empty( $accountData[ 'status' ] ) && $accountData[ 'status' ] === 'error' )
		{
			Helper::response( false, $accountData[ 'error_msg' ] );
		}

		if ( empty( $accountData[ 'data' ] ) || empty( $accountData[ 'data' ][ 'viewer' ][ 'id' ] ) )
		{
			Helper::response( false, fsp__( 'The entered cookies are wrong!' ) );
		}

		return $accountData[ 'data' ][ 'viewer' ];
	}

	public function getAccountNodes ()
	{
		$data = [
			'operationName' => 'StartPage',
			'variables'     => [
				'includeGroupPages'   => false,
				'featuredGroupsCount' => 0,
				'communitiesCount'    => 99,
				'groupsCount'         => 99
			],
			'query'         => 'query StartPage($invitationsCursor: String, $communitiesCursor: String, $groupsCursor: String, $featuredGroupsCount: Int, $communitiesCount: Int, $groupsCount: Int, $includeGroupPages: Boolean = false) {  ...Invitations  ...MyCommunities  ...MyGroups  ...FeaturedGroups  ...CommunityTopics}fragment Invitations on Query {  viewer {    id    invitations(first: 3, after: $invitationsCursor) {      total      pageInfo {        hasNextPage        endCursor        __typename      }      edges {        node {          id          community {            id            name            logo {              urls {                medium                large                __typename              }              __typename            }            claim            memberCount            isMeetup            city {              name              __typename            }            type            links {              public              __typename            }            __typename          }          __typename        }        __typename      }      __typename    }    __typename  }  __typename}fragment MyCommunities on Query {  viewer {    id    communities(first: $communitiesCount, after: $communitiesCursor) {      total      pageInfo {        hasNextPage        endCursor        __typename      }      edges {        node {          id          name          isMeetup          hasNewContent          logo {            urls {              medium              large              __typename            }            __typename          }          links {            public            __typename          }          __typename        }        __typename      }      __typename    }    __typename  }  __typename}fragment MyGroups on Query {  viewer {    id    groupsMemberships(      first: $groupsCount      after: $groupsCursor      states: [APPROVED, BLOCKED]    ) @include(if: $includeGroupPages) {      total      pageInfo {        hasNextPage        endCursor        __typename      }      edges {        node {          group {            id            entityPage {              title              logoMedium: logoImage(dimensions: {width: 70, height: 70, reference: "medium"}) {                url                __typename              }              logoLarge: logoImage(dimensions: {width: 280, height: 280, reference: "large"}) {                url                __typename              }              links {                self                __typename              }              __typename            }            __typename          }          __typename        }        __typename      }      __typename    }    __typename  }  __typename}fragment FeaturedGroups on Query {  viewer {    id    featuredCommunities(first: $featuredGroupsCount) {      edges {        node {          id          name          logo {            urls {              medium              large              __typename            }            __typename          }          claim          memberCount          isMeetup          city {            name            __typename          }          type          links {            public            __typename          }          __typename        }        __typename      }      __typename    }    __typename  }  __typename}fragment CommunityTopics on Query {  communityTopics(limit: 9) {    collection {      title      searchText      recentlyAdded      cityId      filterType      filterValue      images {        large1x        large2x        large3x        __typename      }      __typename    }    __typename  }  __typename}'
		];

		$accountData = self::cmd( 'POST', $this->endpoint, json_encode( $data ), [ 'content-type' => 'application/json' ] );

		if ( ! empty( $accountData[ 'status' ] ) && $accountData[ 'status' ] === 'error' )
		{
			Helper::response( false, $accountData[ 'error_msg' ] );
		}

		if ( empty( $accountData[ 'data' ] ) || empty( $accountData[ 'data' ][ 'viewer' ][ 'id' ] ) )
		{
			Helper::response( false, fsp__( 'The entered cookies are wrong!' ) );
		}

		return $accountData[ 'data' ][ 'viewer' ][ 'communities' ][ 'edges' ];
	}

	public function fetchGroupData ( $nodeId, $groupUrl )
	{
		$forums   = [];
		$response = self::cmd( 'GET', $groupUrl, '', [], false );
		$response = $this->updateGroupUrl( $nodeId, $response );

		if ( is_array( $response ) &&  ! empty( $response[ 'status' ] ) && $response[ 'status' ] === 'error' )
		{
			return $response;
		}

		$forumsHtmlPattern = '/<ul class=\"comm-navigation-dropdown(.*?)<\/ul>/ms';

		preg_match( $forumsHtmlPattern, $response, $matchedForumsHtml );

		$forumsPattern = '/data-forum-id=\"(.+?)\" data-forum-name=\"(.+?)\"/';

		preg_match_all( $forumsPattern, $matchedForumsHtml[ 1 ], $matchedForums );

		if ( ! empty( $matchedForums[ 1 ] ) )
		{
			foreach ( $matchedForums[ 1 ] as $k => $forumId )
			{
				if ( ! isset( $forums[ $forumId ] ) )
				{
					$forums[ $forumId ] = $matchedForums[ 2 ][ $k ];
				}
			}
		}

		$selectedForum = Helper::getCustomSetting( 'xing_group_selected_forum', '', 'node', $nodeId );

		if ( empty( $selectedForum ) || ! isset( $forums[ $selectedForum ] ) )
		{
			Helper::setCustomSetting( 'xing_group_selected_forum', array_key_first( $forums ), 'node', $nodeId );
		}

		Helper::setCustomSetting( 'xing_group_forums', json_encode( $forums ), 'node', $nodeId );
	}

	public function getGroupAuthenticityToken ( $nodeId, $groupUrl )
	{
		$response = self::cmd( 'GET', $groupUrl, '', [], false );

		if ( is_array( $response ) && ! empty( $response[ 'status' ] ) && $response[ 'status' ] === 'error' )
		{
			return $response;
		}

		$response = $this->updateGroupUrl( $nodeId, $response );

		if ( is_array( $response ) && ! empty( $response[ 'status' ] ) && $response[ 'status' ] === 'error' )
		{
			return $response;
		}

		$csrfTokenPattern = '/meta name=\"csrf-token\" content=\"(.+?)\"/';
		$tokenPattern     = '/id=\"new-post-form\"(.*?) name=\"authenticity_token\" value=\"(.+?)\"/';

		preg_match( $csrfTokenPattern, $response, $matchedCsrf );
		preg_match( $tokenPattern, $response, $matchedToken );

		if ( ! empty( $matchedToken[ 2 ] ) )
		{
			return [
				'authenticity_token' => $matchedToken[ 2 ],
				'csrf_token'         => $matchedCsrf[ 1 ]
			];
		}

		return  [
			'status'    => 'error',
			'error_msg' => fsp__( 'You do not have the privilege to share posts on this group!' )
		];
	}

	public function updateGroupUrl ( $nodeId, $response )
	{
		$redirectPattern = '/You are being <a href=\"(.+?)\">redirected<\/a>/';

		preg_match( $redirectPattern, $response, $matchedUrl );

		if ( ! empty( $matchedUrl[ 1 ] ) )
		{
			$response = self::cmd( 'GET', $matchedUrl[ 1 ], '', [], false );

			if ( is_array( $response ) && ! empty( $response[ 'status' ] ) && $response[ 'status' ] === 'error' )
			{
				return $response;
			}

			DB::DB()->update( DB::table( 'account_nodes' ), [ 'screen_name' => $matchedUrl[ 1 ], ], [ 'id' => $nodeId ] );
		}

		return $response;
	}

	public function uploadRequest ( $photo )
	{
		$photoData = file_get_contents( $photo );

		if ( empty( $photoData ) )
		{
			return [
				'status'    => 'error',
				'error_msg' => fsp__( 'The given file path is not valid!' )
			];
		}

		$data      = [
			'operationName' => 'UploadRequest',
			'variables'     => [
				'application' => 'POSTINGS',
				'fileSize'    => strlen( $photoData ),
				'fileType'    => mime_content_type( $photo ) //only supports png/jpeg and gif formats
			],
			'query'         => 'mutation UploadRequest($application: UploadApplication!, $fileSize: Long!, $fileType: String) {  uploadRequest(    input: {application: $application, fileSize: $fileSize, fileType: $fileType}  ) {    success {      id      authToken      url      __typename    }    error {      id      message      __typename    }    __typename  }}'
		];

		$resp = self::cmd( 'POST', $this->endpoint, json_encode( $data ), [ 'content-type' => 'application/json' ] );


		if ( empty( $resp ) || empty( $resp[ 'data' ] ) )
		{
			return [
				'status'    => 'error',
				'error_msg' => ! empty( $resp[ 'errors' ] ) ? $resp[ 'errors' ][ 0 ][ 'message' ] : ( isset( $resp[ 'error_msg' ] ) ? $resp[ 'error_msg' ] : fsp__( 'Couldn\'t upload the image or unknown error!' ) )
			];
		}

		$uploadResp = $resp[ 'data' ][ 'uploadRequest' ];

		if ( ! isset( $uploadResp[ 'success' ] ) )
		{
			if ( ! empty( $resp[ 'data' ][ 'uploadRequest' ][ 'error' ][ 'message' ] ) )
			{
				return [
					'status'    => 'error',
					'error_msg' => $resp[ 'data' ][ 'uploadRequest' ][ 'error' ][ 'message' ]
				];
			}

			return [
				'status'    => 'error',
				'error_msg' => fsp__( 'Couldn\'t upload the image or unknown error!' )
			];
		}

		$result = self::cmd( 'PATCH', $uploadResp[ 'success' ][ 'url' ], $photoData, [
			'Tus-Resumable' => '1.0.0',
			'Upload-Offset' => 0,
			'Authorization' => 'Bearer ' . $uploadResp[ 'success' ][ 'authToken' ],
			'content-type'  => 'application/offset+octet-stream'
		] );

		if ( ! empty( $result ) )
		{
			return [
				'status'    => 'error',
				'error_msg' => substr( $result, 0, 300 )
			];
		}

		return $uploadResp[ 'success' ][ 'id' ];
	}

	private function setClient ()
	{
		$cookieArr = [];

		foreach ( $this->cookies as $k => $v )
		{
			$cookieArr[] = [
				"Name"     => $k,
				"Value"    => $v,
				"Domain"   => '.' . $this->domain,
				"Path"     => "/",
				"Max-Age"  => NULL,
				"Expires"  => NULL,
				"Secure"   => FALSE,
				"Discard"  => FALSE,
				"HttpOnly" => FALSE,
				"Priority" => "HIGH"
			];
		}

		$cookieJar = new CookieJar( FALSE, $cookieArr );

		$this->client = new Client( [
			'cookies'         => $cookieJar,
			'proxy'           => empty( $this->proxy ) ? NULL : $this->proxy,
			'verify'          => FALSE,
			'http_errors'     => FALSE,
			'headers'         => [
				'User-Agent'   => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:75.0) Gecko/20100101 Firefox/75.0',
				'X-CSRF-Token' => $this->cookies[ 'xing_csrf_token' ],
				'Host'         => $this->domain
			 ]
		] );
	}

	public function refetchAccount ( $accountId )
	{
		$groupIDsList = $this->getAccountNodes();
		$getNodes     = DB::DB()->get_results( DB::DB()->prepare( 'SELECT id, node_id FROM ' . DB::table( 'account_nodes' ) . ' WHERE account_id = %d', [ $accountId ] ), ARRAY_A );
		$myNodes      = [];

		foreach ( $getNodes as $node )
		{
			$myNodes[ $node[ 'id' ] ] = $node[ 'node_id' ];
		}

		if ( ! empty( $groupIDsList ) )
		{
			foreach ( $groupIDsList as $node )
			{
				$node = $node[ 'node' ];
				$url  = ! $node[ 'isMeetup' ] ? $node[ 'links' ][ 'public' ] : str_replace( 'about', 'posts', $node[ 'links' ][ 'public' ] );

				if ( ! in_array( $node[ 'id' ], $myNodes ) )
				{

					error_log( 's' );
					DB::DB()->insert( DB::table( 'account_nodes' ), [
						'blog_id'     => Helper::getBlogId(),
						'user_id'     => get_current_user_id(),
						'account_id'  => $accountId,
						'driver'      => 'xing',
						'node_type'   => 'group',
						'node_id'     => $node[ 'id' ],
						'name'        => $node[ 'name' ],
						'cover'       => $node[ 'logo' ][ 'urls' ][ 'medium' ],
						'screen_name' => $url
					] );

					error_log( 'h' );
					$id = DB::DB()->insert_id;
				}
				else
				{
					DB::DB()->update( DB::table( 'account_nodes' ), [
						'name'        => $node[ 'name' ],
						'cover'       => $node[ 'logo' ][ 'urls' ][ 'medium' ],
						'screen_name' => $url
					], [
						'account_id' => $accountId,
						'node_id'    => $node[ 'id' ]
					] );

					$id = array_search( $node[ 'id' ], $myNodes );

					unset( $myNodes[ $id ] );
				}

				$this->fetchGroupData( $id, $url );
			}
		}

		if ( ! empty( $myNodes ) )
		{
			DB::DB()->query( 'DELETE FROM ' . DB::table( 'account_nodes' ) . ' WHERE id IN (' . implode( ',', array_keys( $myNodes ) ) . ')' );
			DB::DB()->query( 'DELETE FROM ' . DB::table( 'account_node_status' ) . ' WHERE node_id IN (' . implode( ',', array_keys( $myNodes ) ) . ')' );

			foreach ( $myNodes as $k => $v )
			{
				Helper::deleteCustomSettings( 'node', $k );
			}
		}

		return [ 'status' => TRUE ];
	}

	public function updateCookies ( $id, $profileId )
	{
		$myInfo = $this->getAccountData();

		if ( ! isset( $myInfo[ 'id' ] ) || ( $profileId !== $myInfo[ 'id' ]) )
		{
			return FALSE;
		}

		$dataSQL = [
			'proxy'     => $this->proxy,
			'options'   => json_encode( $this->cookies ),
			'status'    => NULL,
			'error_msg' => NULL
		];

		DB::DB()->update( DB::table( 'accounts' ), $dataSQL, [ 'id' => $id ] );

		return TRUE;
	}

	public function checkAccount()
	{
		$data = [
			'operationName' => 'xingFrameQuery',
			'variables'     => [],
			'query'         => 'query xingFrameQuery {  viewer {    id    webTrackingData {      PropHashedUserId      __typename    }    xingId {      academicTitle      birthday      displayName      displayFlag      userFlags {        displayFlag        __typename      }      firstName      gender      globalId      id      lastName      pageName      profileImage(size: [SQUARE_128]) {        url        __typename      }      profileOccupation {        occupationOrg        occupationTitle        __typename      }      occupations {        headline        subline        __typename      }      __typename    }    features {      isXingEmployee      isJobsPoster      isAdmasterUser      isBrandPageCollaborator      isBasic      isPremium      isExecutive      isSales      hasProJobsMembership      isCraUser      isSeatManagerAdmin      showProbusinessInNavigation      showUpsellHint      showJobSeekerBoneyardUpsellHint      showPremiumBoneyardUpsellHint      hasCoachProfile      hasNewSettings      isBrandManagerEditor      __typename    }    featureSwitches    loginState    __typename  }}'
		];

		$response = self::cmd( 'POST', $this->endpoint, json_encode( $data ), [ 'content-type' => 'application/json' ] );

		if ( ! $response )
		{
			return [ 'error' => TRUE ];
		}
		else if ( isset( $response[ 'error_msg' ] ) )
		{
			return [
				'error'     => TRUE,
				'error_msg' => $response[ 'error_msg' ]
			];
		}

		return $response;
	}
}
